/*============================================================================
  File:     DynamicStringExecution v. sp_executesql

  Summary:  What are the differences in the way that queries are submitted?
  
  Date:     October 2011

  SQL Server Version: 2005/2008
------------------------------------------------------------------------------
  Written by Kimberly L. Tripp, SQLskills
  
  For more scripts and sample code, check out 
    http://www.SQLskills.com

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

-- Make sure this is NOT set, otherwise, be sure to turn it off for this
-- demo. However, I generally recommend that it's set in production!
-- And, this is a server-wide setting - do NOT "play" with this script
-- in production!!

SELECT * 
FROM sys.configurations
WHERE name = N'optimize for ad hoc workloads'
go

--EXEC sp_configure 'optim', 0  -- actual option: 'optimize for ad hoc workloads'
--                                 -- However, sp_configure only needs enough of the option
--                                 -- to make it unique.
--go

--reconfigure
--go

USE credit
go

-- Some quick setup
UPDATE member	
SET lastname = 'Tripp' 
WHERE member_no = 1234
go

CREATE INDEX MemberLastName 
ON dbo.member (lastname)
go

sp_helpindex member
go

--------------------------------------------------
-- Can we see what's in cache?
--
-- Simple proc to quickly see a subset of
-- what statements are in cache - with their
-- plan.
--------------------------------------------------
CREATE PROCEDURE QuickCheckOnCache
    @StringToFind   nvarchar(4000)
AS
SELECT st.text, qs.EXECUTION_COUNT, qs.*, p.* 
FROM sys.dm_exec_query_stats AS qs 
	CROSS APPLY sys.dm_exec_sql_text(sql_handle) st
	CROSS APPLY sys.dm_exec_query_plan(plan_handle) p
WHERE st.text LIKE @StringToFind
ORDER BY 1, qs.EXECUTION_COUNT DESC;
go

-- Make it a bit easier (more isolated) 
-- to review what's in cache and to clear
-- any plans that might have ended up there
-- that could influence the results of this
-- scripts:
DBCC FREEPROCCACHE
go

-- These are great for review:
SET STATISTICS IO ON
SET STATISTICS TIME ON
-- Also, turn Actual Graphical Showplan ON
go

--------------------------------------------------
-- Execute a few statements and review how
-- they're handled in cache
 
-- Parameterized? Safe? Unsafe?
--------------------------------------------------
go

SELECT * 
FROM dbo.member 
WHERE member_no = 258
go

SELECT * 
FROM dbo.member 
WHERE member_no = 34567
go

SELECT * 
FROM dbo.member 
WHERE member_no = 33
go

EXEC QuickCheckOnCache '%member%'
go

-- NOTES: The first 3 are deemed safe.

SELECT * 
FROM dbo.member 
WHERE lastname = 'Tripp'
go

SELECT * 
FROM dbo.member 
WHERE lastname LIKE 'Tripp'
go

SELECT * 
FROM dbo.member 
WHERE lastname LIKE 'Anderson'
go

SELECT * 
FROM dbo.member 
WHERE lastname LIKE '%e%'
go

EXEC QuickCheckOnCache '%member%'

go

-- NOTES: All four of these are all deemed unsafe.

-----------------------------------------------------------------------------
-- How do things change with sp_executesql or EXEC?
-----------------------------------------------------------------------------

--------------------------------------------------
-- sp_executesql with a "safe" statement
--------------------------------------------------
DECLARE @ExecStr    nvarchar(4000)
SELECT @ExecStr = 'SELECT * FROM dbo.member WHERE member_no = @mno'
EXEC sp_executesql @ExecStr,
                      N'@mno int',
                      1234
go
                      
DECLARE @ExecStr    nvarchar(4000)
SELECT @ExecStr = 'SELECT * FROM dbo.member WHERE member_no = @mno'
EXEC sp_executesql @ExecStr,
                      N'@mno int',
                      12
go

DECLARE @ExecStr    nvarchar(4000)
SELECT @ExecStr = 'SELECT * FROM dbo.member WHERE member_no = @mno'
EXEC sp_executesql @ExecStr,
                      N'@mno int',
                      76896
go

EXEC QuickCheckOnCache '%member%'
go

-- NOTES: Only ONE statement is placed in cache. And, notice that
-- it's strongly typed. For a safe statement this is beneficial.

--------------------------------------------------
-- sp_executesql with a different (and unsafe) statement
--------------------------------------------------
DECLARE @ExecStr    nvarchar(4000)
SELECT @ExecStr = 'SELECT m.* FROM dbo.member AS m WHERE m.lastname like @lastname'
EXEC sp_executesql @ExecStr,
                      N'@lastname varchar(15)',
                      'Tripp'
go
                      
DECLARE @ExecStr    nvarchar(4000)
SELECT @ExecStr = 'SELECT m.* FROM dbo.member AS m WHERE m.lastname like @lastname'
EXEC sp_executesql @ExecStr,
                      N'@lastname varchar(15)',
                      'Anderson'
go

DECLARE @ExecStr    nvarchar(4000)
SELECT @ExecStr = 'SELECT m.* FROM dbo.member AS m WHERE m.lastname like @lastname OPTION (RECOMPILE)'
EXEC sp_executesql @ExecStr,
                      N'@lastname varchar(15)',
                      '%e%'
go

EXEC QuickCheckOnCache '%member%'
go

--------------------------------------------------
-- EXECUTE with safe statement
--------------------------------------------------
DECLARE @ExecStr    nvarchar(4000),
		@MemberNo	int
SELECT @MemberNo = 1567
-- This first version is what's most likely for people to write. However, you'll end up with a plan per data type
SELECT @ExecStr = N'SELECT m.* FROM dbo.member AS m WHERE m.member_no = ' + convert(nvarchar(10), @MemberNo) 
-- This second version is better as the variable is strongly typed. If the statement is "safe" SQL Server will only have ONE plan to use/re-use.
--SELECT @ExecStr = N'SELECT m.* FROM dbo.member AS m WHERE m.member_no = convert(int, ' + convert(nvarchar(10), @MemberNo) + N')'
SELECT @ExecStr
EXEC (@ExecStr)
go
                      
DECLARE @ExecStr    nvarchar(4000),
		@MemberNo	int
SELECT @MemberNo = 17
SELECT @ExecStr = N'SELECT m.* FROM dbo.member AS m WHERE m.member_no = ' + convert(nvarchar(10), @MemberNo) 
--SELECT @ExecStr = N'SELECT m.* FROM dbo.member AS m WHERE m.member_no = convert(int, ' + convert(nvarchar(10), @MemberNo) + N')'
SELECT @ExecStr
EXEC (@ExecStr)
go

DECLARE @ExecStr    nvarchar(4000),
		@MemberNo	int
SELECT @MemberNo = 67890
SELECT @ExecStr = N'SELECT m.* FROM dbo.member AS m WHERE m.member_no = ' + convert(nvarchar(10), @MemberNo) 
--SELECT @ExecStr = N'SELECT m.* FROM dbo.member AS m WHERE m.member_no = convert(int, ' + convert(nvarchar(10), @MemberNo) + N')'
SELECT @ExecStr
EXEC (@ExecStr)
go

EXEC QuickCheckOnCache '%member_no%'
go
--------------------------------------------------
-- EXECUTE with an unsafe statement
--------------------------------------------------
DECLARE @ExecStr    nvarchar(4000),
		@Lastname	varchar(15)
SELECT @Lastname = 'Tripp'
SELECT @ExecStr = N'SELECT m.* FROM dbo.member AS m WHERE m.lastname LIKE convert(varchar(15), ' 
    + QUOTENAME(@lastname, '''') + N')'
--SELECT @ExecStr
-- Using REPLACE instead of QUOTENAME. 
-- QUOTENAME is limited to nvarchar(128) 
-- for longer strings - you must use REPLACE:
--SELECT @ExecStr = N'SELECT * FROM dbo.member WHERE lastname LIKE convert(varchar(15), ''' + replace(@lastname, '''', '''''') + N''')'
SELECT @ExecStr
EXEC (@ExecStr)
go
                      
DECLARE @ExecStr    nvarchar(4000),
		@Lastname	varchar(15)
SELECT @Lastname = 'Anderson'
SELECT @ExecStr = N'SELECT m.* FROM dbo.member AS m WHERE m.lastname LIKE convert(varchar(15), ' + QUOTENAME(@lastname, '''') + N')'
--SELECT @ExecStr
EXEC (@ExecStr)
go

DECLARE @ExecStr    nvarchar(4000),
		@Lastname	varchar(15)
SELECT @Lastname = '%e%'
SELECT @ExecStr = N'SELECT m.* FROM dbo.member AS m WHERE m.lastname LIKE convert(varchar(15), ' + QUOTENAME(@lastname, '''') + N')'
--SELECT @ExecStr
EXEC (@ExecStr)
go

EXEC QuickCheckOnCache '%varchar(15)%'
go