/*============================================================================
  File:     Multipurpose Procs.sql

  Summary:  This script shows the idea behind dynamic string execution
            and its benefits.

  Date:     March 2012

  SQL Server Version: 2005/2008/2012
------------------------------------------------------------------------------
  Written by Kimberly L. Tripp

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE credit
go

SET STATISTICS IO ON
-- Turn Graphical Showplan ON (Ctrl+K)

-- Add an index to SEEK for LastNames
CREATE INDEX MemberFirstName ON dbo.Member(Firstname)
go

-- Add an index to SEEK for LastNames
CREATE INDEX MemberLastName ON dbo.Member(Lastname)
go

UPDATE dbo.member
	SET lastname = 'Tripp'
	WHERE member_no = 1234
go

UPDATE dbo.member
	SET firstname = 'Kimberly'
	WHERE member_no = 2479
go

IF OBJECTPROPERTY(object_id('GetMemberInfoParam'), 'IsProcedure') = 1
	DROP PROCEDURE GetMemberInfoParam
go

CREATE PROC GetMemberInfoParam
	@Lastname	varchar(15) = NULL,
	@Firstname	varchar(15) = NULL,
	@member_no	int = NULL
AS
SELECT m.* FROM dbo.member AS m
WHERE (m.lastname = @lastname OR @lastname IS NULL)
	AND (m.member_no = @member_no OR @member_no IS NULL)
	AND (m.firstname LIKE @firstname OR @firstname IS NULL)
go

--sp_recompile GetMemberInfoParam	

EXEC GetMemberInfoParam	@Lastname = 'Tripp' 
go

EXEC GetMemberInfoParam	@Firstname = 'Kimberly'
go

EXEC GetMemberInfoParam	@Member_no = 9912
go

-- What about with OPTION (RECOMPILE)??
ALTER PROC GetMemberInfoParam
	@Lastname	varchar(15) = NULL,
	@Firstname	varchar(15) = NULL,
	@member_no	int = NULL
AS
SELECT m.* FROM dbo.member AS m
WHERE (m.lastname LIKE @lastname OR @lastname IS NULL)
	AND (m.member_no = @member_no OR @member_no IS NULL)
	AND (m.firstname LIKE @firstname OR @firstname IS NULL)
	OPTION (RECOMPILE) -- Note: this works MUCH better in 2012 but it's still not ideal (or consistent across versions)
go

EXEC GetMemberInfoParam	@Lastname = 'Tripp' 
go

EXEC GetMemberInfoParam	@Firstname = 'Kimberly'
go

EXEC GetMemberInfoParam	@Member_no = 9912
go

CREATE PROC GetMemberInfoParam2
	@Lastname	varchar(15) = NULL,
	@Firstname	varchar(15) = NULL,
	@member_no	int = NULL
AS
DECLARE @ExecStr	nvarchar(1000)
SELECT @ExecStr = N'SELECT m.* FROM dbo.member AS m WHERE 1=1 ' 

IF @LastName IS NOT NULL
	SELECT @ExecStr = @ExecStr + N'AND m.lastname LIKE convert(varchar(15), ' + QUOTENAME(@lastname, '''') + N') '
IF @FirstName IS NOT NULL
	SELECT @ExecStr = @ExecStr + N'AND m.firstname LIKE convert(varchar(15), ' + QUOTENAME(@firstname, '''') + N') '
IF @Member_no IS NOT NULL
	SELECT @ExecStr = @ExecStr + N'AND m.member_no = convert(int, ' + convert(varchar(5), @member_no) + N') '

SELECT (@ExecStr)
EXEC(@ExecStr)
go

EXEC GetMemberInfoParam2	@Lastname = 'Tripp'
    , @FirstName = 'Kimberly' 
go

EXEC GetMemberInfoParam2	@Firstname = 'Kimberly' 
go

EXEC GetMemberInfoParam2	@Firstname = 'Kimberly', @Member_no = 842 
go

EXEC GetMemberInfoParam2	@Member_no = 9912 
go

EXEC GetMemberInfoParam2	@Lastname = 'Florini', @Member_no = 9912 
go

-- Instead - using sp_executesql
-- if the majority of parameters are safe then sp_executesql can be great...
-- if a few of the parameters are unsafe then add OPTION (RECOMPILE)
-- And, if there's a single parameter that will guarantee a highly
-- selective set you can turn off the @Recompile FLAG by placing it
-- add the end of the list!
CREATE PROC GetMemberInfoParam3
(	@Lastname	varchar(15) = NULL,
	@Firstname	varchar(15) = NULL,
	@member_no	int = NULL)
AS
IF @LastName IS NULL AND @FirstName IS NULL AND @Member_no IS NULL
	RAISERROR ('You must supply at least one parameter.', 16, -1)

DECLARE @ExecStr	nvarchar(4000),
        @Recompile  bit
SELECT @ExecStr = N'SELECT m.* FROM dbo.member AS m WHERE 1=1' 

IF @LastName IS NOT NULL
BEGIN
	SELECT @ExecStr = @ExecStr + N' AND m.lastname LIKE @Lname' 
	SELECT @Recompile = 1
END

IF @FirstName IS NOT NULL
BEGIN
	SELECT @ExecStr = @ExecStr + N' AND m.firstname LIKE @Fname'
	SELECT @Recompile = 1
END

IF @Member_no IS NOT NULL
BEGIN
	SELECT @ExecStr = @ExecStr + N' AND m.member_no = @Memno'
	SELECT @Recompile = 0
END
	
IF @Recompile = 1
    SELECT @ExecStr = @ExecStr + N' OPTION(RECOMPILE)'
    
SELECT @ExecStr, @Lastname, @Firstname, @member_no

EXEC sp_executesql @ExecStr
    , N'@Lname varchar(15), @Fname varchar(15), @Memno int'
	, @Lname = @Lastname
	, @Fname = @Firstname
	, @Memno = @Member_no
go

EXEC GetMemberInfoParam3	'Tripp', 'Kimberly' 
go

EXEC GetMemberInfoParam3	@Firstname = 'Kimberly' 
go

EXEC GetMemberInfoParam3	@Firstname = 'Kimberly', @Member_no = 842 
go

EXEC GetMemberInfoParam3	@Member_no = 9912 
go

EXEC GetMemberInfoParam3	@Lastname = 'Florini', @Member_no = 9912 
go

-- This is another example of a frequently asked version...
-- This does not create an optimal plan either.
CREATE PROC GetMemberInfoParam4
	@Lastname	varchar(30) = NULL,
	@Firstname	varchar(30) = NULL,
	@member_no	int = NULL
AS
SELECT m.* FROM dbo.member AS m
WHERE m.lastname =
	CASE WHEN @lastname IS NULL THEN m.lastname
			ELSE @lastname
	END
	AND m.firstname =
	CASE WHEN @firstname IS NULL THEN m.firstname
			ELSE @firstname
	END
	AND m.member_no =
	CASE WHEN @member_no IS NULL THEN m.member_no
			ELSE @member_no
	END
go

EXEC GetMemberInfoParam4	@Lastname = 'test', @FirstName = 'Kimberly' 
go

EXEC GetMemberInfoParam4	@Firstname = 'Kimberly' with recompile
go
exec GetMemberInfoParam4	@Firstname = 'Kimberly', @Member_no = 842  with recompile
go
exec GetMemberInfoParam4	@Member_no = 9912 with recompile
go
exec GetMemberInfoParam4	@Lastname = 'Florini', @Member_no = 9912 with recompile
go