/*============================================================================
  File:     Parameters and filters.sql

  Summary:  Filtered Indexes (and filtered stats) are NOT accessible to
            forced parameterization OR parameterized plans (in procs).
            But, there is a work-around.

  Date:     June 2012

  SQL Server Version: SQL Server 2008 R2
------------------------------------------------------------------------------
  Written by Kimberly L. Tripp & Paul S. Randal, SQLskills.com

  For more scripts and sample code, check out http://www.SQLskills.com

  This script is intended as a supplement to the SQL Server 2008 Jumpstart or
  Metro training.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

SET NOCOUNT ON
go

USE JunkDB
go

IF OBJECTPROPERTY(object_id('[dbo].[TestFilteredIndex]'), 'IsUserTable') IS NOT NULL
    DROP TABLE [dbo].[TestFilteredIndex]
go

-- Create a test table
CREATE TABLE [dbo].[TestFilteredIndex]
(
       [id] [int] IDENTITY(1,1) NOT NULL,
       [date] [date] NULL,
CONSTRAINT [PK_IF] PRIMARY KEY CLUSTERED 
(
       [id] ASC
))
GO
-- Insert some rows
INSERT INTO [dbo].[TestFilteredIndex] ([date]) VALUES ('20120601')
GO 100
INSERT INTO [dbo].[TestFilteredIndex] ([date]) VALUES ('20120615')
GO 1000
INSERT INTO [dbo].[TestFilteredIndex] ([date]) VALUES ('20120630')
GO 10000

-- See the data/distribution
SELECT COUNT(*), [DATE]
FROM [dbo].[TestFilteredIndex]
GROUP BY [DATE]
go

-- Create a filtered index
CREATE NONCLUSTERED INDEX [FilteredIndexONTest] ON [dbo].[TestFilteredIndex]
([date] ASC)  WHERE  date < '2012-06-13'

GO
-- Check the query plan for adhoc queries

SELECT [date]
  FROM [dbo].[TestFilteredIndex]
  WHERE date < '2012-06-06' -- uses the FI
go

SELECT[date]
  FROM [dbo].[TestFilteredIndex]
  WHERE date < '2012-06-10' -- uses the FI
go

SELECT[date]
  FROM [dbo].[TestFilteredIndex]
  WHERE date < '2012-06-20' -- does not use the FI
go

--------------------------------------------
-- What about a stored procedure:
--------------------------------------------
DROP PROCEDURE Test
go

CREATE PROCEDURE test (@d date) 
AS
BEGIN
  SELECT [date]
  FROM [dbo].[TestFilteredIndex] 
  WHERE date < @d
END
GO

DBCC FREEPROCCACHE
go

EXECUTE test '20120606'
EXECUTE test '20120610'
EXECUTE test '20120620'
go

-- NONE of them use the filtered index!

--------------------------------------------
-- What about with RECOMPILE??
--------------------------------------------

ALTER PROCEDURE test (@d date) WITH RECOMPILE
AS
BEGIN
  SELECT [date]
  FROM [dbo].[TestFilteredIndex] 
  WHERE date < @d
END
GO

DBCC FREEPROCCACHE
go

EXECUTE test '20120606'
EXECUTE test '20120610'
EXECUTE test '20120620'
go
-- NONE of them use the filtered index!

--------------------------------------------
-- What about OPTION RECOMPILE??
--------------------------------------------

-- NOTE: This WORKS in 2012 and 2008 NOT 2008R2
-- AND, it's OPTION RECOMPILE that works NOT WITH RECOMPILE
ALTER PROCEDURE test (@d date)
AS
BEGIN
  SELECT [date]
  FROM [dbo].[TestFilteredIndex] 
  WHERE date < @d
  OPTION (RECOMPILE)
END
GO

DBCC FREEPROCCACHE
go

EXECUTE test '20120606'
EXECUTE test '20120610'
EXECUTE test '20120620'
go

-- NONE of them use the filtered index!


--------------------------------------------
-- What about with RECOMPILE AND RECOMPILE??
--------------------------------------------

ALTER PROCEDURE test (@d date)
WITH RECOMPILE
AS
BEGIN
  SELECT [date]
  FROM [dbo].[TestFilteredIndex] 
  WHERE date < @d
  OPTION (RECOMPILE)
END
GO

DBCC FREEPROCCACHE
go

EXECUTE test '20120606'
EXECUTE test '20120610'
EXECUTE test '20120620'
go

-- NONE of them use the filtered index!



--------------------------------------------
-- How do you resolve this??
-- DSE
--------------------------------------------

ALTER PROCEDURE test (@d date) 
AS
BEGIN
DECLARE @ExecStr    nvarchar(1000)
SELECT @ExecStr = N'SELECT [date] FROM [dbo].[TestFilteredIndex] WHERE date < ''' + convert(nvarchar, @d) + ''''
EXEC (@ExecStr)
END
GO

DBCC FREEPROCCACHE
go

EXECUTE test '20120606'
EXECUTE test '20120610'
EXECUTE test '20120620'
go

--------------------------------------------
-- What about SQL Injection and DSE?
--------------------------------------------

CREATE USER User_test 
WITHOUT LOGIN
go
 
GRANT SELECT ON [dbo].[TestFilteredIndex] TO User_test
go
 
ALTER PROCEDURE test
(
      @d date
)
WITH EXECUTE AS N'User_test'
AS
DECLARE @ExecStr    nvarchar(1000)
SELECT @ExecStr = N'SELECT [date] FROM [dbo].[TestFilteredIndex] WHERE date < ''' + convert(nvarchar, dateadd(dd, 0, @d)) + ''''
EXEC (@ExecStr)
go

DBCC FREEPROCCACHE
go

-- Turn OFF showplan first (User_test doesn't have rights)
-- OR, grant rights

GRANT SHOWPLAN TO User_test
go

EXECUTE test '20120606'
EXECUTE test '20120610'
EXECUTE test '20120620'
go

--------------------------------------------
-- What if you can't give them showplan perms?
--------------------------------------------

SELECT st.text, qs.query_hash, qs.query_plan_hash, 
        qs.EXECUTION_COUNT, qs.plan_handle, 
        qs.statement_start_offset, qs.*, qp.* 
FROM sys.dm_exec_query_stats AS qs 
	CROSS APPLY sys.dm_exec_sql_text(sql_handle) AS st
	CROSS APPLY sys.dm_exec_query_plan(plan_handle) AS qp
WHERE st.text LIKE '%WHERE date < %'
	AND (st.text NOT LIKE '%syscacheobjects%' OR st.text NOT LIKE '%SELECT%cp.objecttype%')
ORDER BY 1, qs.EXECUTION_COUNT DESC;
go

EXECUTE test '20120601' 
go

-- Run an adhoc version to get the query hash and query plan hash
-- You can check/compare the query plan hash for the statement
SELECT [date] FROM [dbo].[TestFilteredIndex] 
AS ADHOCVER WHERE date < '20120606'
go

SELECT st.text, qs.query_hash, qs.query_plan_hash, 
        qs.EXECUTION_COUNT, qs.plan_handle, 
        qs.statement_start_offset, qs.*, qp.* 
FROM sys.dm_exec_query_stats AS qs 
	CROSS APPLY sys.dm_exec_sql_text(sql_handle) AS st
	CROSS APPLY sys.dm_exec_query_plan(plan_handle) AS qp
WHERE st.text LIKE '%ADHOCVER%'
	AND (st.text NOT LIKE '%syscacheobjects%' OR st.text NOT LIKE '%SELECT%cp.objecttype%')
ORDER BY 1, qs.EXECUTION_COUNT DESC;
go

